#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2021/9/10 10:53
# @Author  : Adyan
# @File    : proxy.py


import json
import logging
import random
import re
import time
import requests

cache = {}
limit_times = [10]


def stat_called_time(func):
    def _called_time(*args, **kwargs):
        key = func.__name__
        if key in cache.keys():
            cache[key][0] += 1
        else:
            call_times = 1
            cache[key] = [call_times, time.time()]
        if cache[key][0] <= limit_times[0]:
            res = func(*args, **kwargs)
            cache[key][1] = time.time()
            return res
        else:
            return None

    return _called_time


class ProxyMeta(type):
    def __new__(mcs, name, bases, attrs):
        attrs['__Crawl_Func__'] = []
        for k, v in attrs.items():
            if 'crawl_' in k:
                attrs['__Crawl_Func__'].append(k)

        return type.__new__(mcs, name, bases, attrs)


class ProxyGetter(metaclass=ProxyMeta):
    """
    :sample code:
        ProxyGetter(
         url='Obtain the proxy IP address',
         name='The name of the method called',
         add_whitelist='Add a whitelist address'
         del_whitelist='The whitelist address is deleted'
     ).get_proxies()
    """

    def __init__(
            self, url, name,
            add_whitelist=None,
            del_whitelist=None
    ):
        self.url = url
        self.name = name
        self.add_whitelist = add_whitelist
        self.del_whitelist = del_whitelist

    def get_proxies(self):
        """
        通过方法名调用方法，获取代理
        :param callfunc:
        :return:
        """
        proxies = []
        try:
            res = eval(f'self.{self.name}()')
            if isinstance(res, dict):
                return res
            for proxy in res:
                proxies.append(proxy)
        except:
            pass
        if proxies:
            return proxies

    @stat_called_time
    def jingling_ip(self):
        ip_list = []
        res = json.loads(requests.get(self.url).text)
        if not res.get("data"):
            msg = res.get('msg')
            if "限制" in msg:
                time.sleep(random.randint(1, 3))
            if '登' in msg:
                limit_times[0] += 1
                _ip = re.findall('(.*?)登', res.get("msg"))
                requests.get(self.add_whitelist % _ip[0], timeout=10)
            res = json.loads(requests.get(self.url).text)

        if res.get("data"):
            for item in res.get("data"):
                ip_list.append("https://" + item['IP'])
        else:
            print(res)
            logging.info(res)
        if ip_list:
            return ip_list
        else:
            return res

    @stat_called_time
    def taiyang_ip(self):
        time.sleep(random.randint(2, 4))
        ip_list = []
        res = json.loads(requests.get(self.url).text)
        if not res.get("data"):
            msg = res.get("msg")
            if '请2秒后再试' in msg:
                limit_times[0] += 1
            if '请将' in msg:
                _ip = msg.split("设")[0][2:]
                print(requests.get(self.add_whitelist % _ip).text)
            if '当前IP' in msg:
                _ip = msg.split("：")[1]
                print(requests.get(self.add_whitelist % _ip).text)
            res = json.loads(requests.get(self.url).text)
        # print(r)

        if res.get("data"):
            for item in res.get("data"):
                ip_list.append(f"https://{item['ip']}:{item['port']}")
        else:
            logging.info(res)

        if ip_list:
            return ip_list
        else:
            return res

    @stat_called_time
    def zhima_ip(self):
        pass

    @stat_called_time
    def source_proxy(self):
        ip_list = [1]
        if ip_list:
            return ip_list
        return ip_list



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2021/11/26 14:20
# @Author  : Adyan
# @File    : __init__.py.py

from .Rabbit_conn import *
from .Redis_conn import *
from .monitor_rabbit import *
from .my_middleware import *
from .MyUtils import *
from .Mongo_conn import *
from .proxy import *
from .H5_TK import *


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2021/6/29 19:09
# @Author  : Adyan
# @File    : MyUtils.py

import json
import re
import time
import pytz
import random

from datetime import datetime
from faker import Faker

from .Mongo_conn import MongoPerson

fake = Faker()
cntz = pytz.timezone("Asia/Shanghai")


class ReDict:

    @classmethod
    def string(
            cls,
            re_pattern: dict,
            string_: str,
    ):
        if string_:
            return {
                key: cls.compute_res(
                    re_pattern=re.compile(scale),
                    string_=string_.translate(
                        {
                            ord('\t'): '', ord('\f'): '',
                            ord('\r'): '', ord('\n'): '',
                            ord(' '): '',
                        })
                )
                for key, scale in re_pattern.items()
            }

    @classmethod
    def compute_res(
            cls,
            re_pattern: re,
            string_=None
    ):
        data = [
            result.groups()[0]
            for result in re_pattern.finditer(string_)
        ]
        if data:
            try:
                return json.loads(data[0])
            except:
                return data[0]
        else:
            return None


class GoodsCopyUtils:

    @classmethod
    def time_cycle(
            cls,
            times,
            int_time=None
    ):
        """
        入库时间规整
        :param times: string - 字符串时间
        :param int_time: True and False  - 获时间戳
        :return:
        """
        if int_time:
            return int(time.mktime(time.strptime(times, "%Y-%m-%d")))
        if type(times) is str:
            times = int(time.mktime(time.strptime(times, "%Y-%m-%d %H:%M:%S")))
        return str(datetime.fromtimestamp(times, tz=cntz))

    @classmethod
    def merge_dic(
            cls,
            dic: dict,
            lst: list
    ):
        """
        合并多个dict
        :param dic: dict - 主dict
        :param lst: list - 多个字典列表方式传入
        :return:
        """
        for d in lst:
            for k, v in d.items():
                if v:
                    dic[k] = v
        return dic

    @classmethod
    def is_None(
            cls,
            dic: dict,
    ) -> dict:
        """
        :param dic: dict
        :return: 返回字典中值是None的键值对
        """
        return {
            k: v
            for k, v in dic.items()
            if not v
        }

    @classmethod
    def find(
            cls, target: str,
            dictData: dict,
    ) -> list:
        queue = [dictData]
        result = []
        while len(queue) > 0:
            data = queue.pop()
            for key, value in data.items():
                if key == target:
                    result.append(value)
                elif isinstance(value, dict):
                    queue.append(value)
        if result:
            return result[0]


class Headers:

    def headers(self, referer=None):
        while True:
            user_agent = fake.chrome(
                version_from=63, version_to=80, build_from=999, build_to=3500
            )
            if "Android" in user_agent or "CriOS" in user_agent:
                continue
            else:
                break
        if referer:
            return {
                "user-agent": user_agent,
                "referer": referer,
            }
        return {
            "user-agent": user_agent,
        }


class Cookies(object):

    def __init__(self, db_name, mongo_config):
        self.mongo_conn = MongoPerson(db_name, 'cookie', mongo_config).test()

    def cookie(self):
        return random.choice(list(self.mongo_conn.find()))


class SKU:
    def skuMap(self, param):
        data = [
            self.style(i.get("data_value"), i.get("skuName"), i.get("style"), i.get("cls"))
            for i in param
            if i.get("data_value")
        ]
        return self.sku(data), [s[1] for s in data]

    def style(self, data_value, skuName, style, cls):
        try:
            pid = data_value[0].split(":")[0]
        except:
            pid = None

        if style:
            styles = []
            for k, v in zip(
                    [
                        {'data_value': k, 'skuName': f'{cls[0]}--{v}'}
                        for k, v in zip(data_value, skuName)
                    ], style
            ):
                k.update({"image": v.replace("background:url(", '').replace(") center no-repeat;", '')})
                styles.append(k)
            props = {
                "values": [
                    {
                        'vid': k.split(":")[1],
                        'name': v,
                        'image': self.props_image(styles, k)
                    }
                    for k, v in zip(data_value, skuName)
                ],
                "name": cls[0],
                "pid": pid,
            }

        else:
            styles = [
                {'data_value': k, 'skuName': f'{cls[0]}--{v}'}
                for k, v in zip(data_value, skuName)
            ]
            props = {
                "values": [
                    {
                        'vid': k.split(":")[1],
                        'name': v,
                        'image': self.props_image(styles, k)
                    }
                    for k, v in zip(data_value, skuName)
                ],
                "name": cls[0],
                "pid": pid,
            }
        if pid:
            return styles, props
        else:
            return styles, None

    def sku(self, lis):
        results = []
        for data in lis:
            p1 = data[1]["values"]
            p2 = []
            for i, s in enumerate(data[0]):
                s.update({"image": p1[i]["image"]})
                p2.append(s)
            results.append(p2)

        shape = [len(v) for v in results]
        offsets = [0] * len(shape)
        has_next = True
        values = []

        while has_next:
            record = [results[i][off] for i, off in enumerate(offsets)]
            if not record:
                break
            res = self.format_data(record)
            values.append(res)

            for i in range(len(shape) - 1, -1, -1):
                if offsets[i] + 1 >= shape[i]:
                    offsets[i] = 0  # 重置并进位
                    if i == 0:
                        has_next = False  # 全部占满，退出
                else:
                    offsets[i] += 1
                    break

        return values

    def format_data(self, ls, ):
        p1 = []
        p2 = []
        p3 = []

        for li in ls:
            p1.append(list(li.values())[0])
            p2.append(list(li.values())[1])
            p3.append(list(li.values())[2])

        date_value = f";{';'.join(p1)};"
        skuName = f";{';'.join(p2)};"
        image = [i for i in p3 if i]

        return {"data_value": date_value, "skuName": skuName, "image": image[0] if image else None}

    def props_image(self, style, k):
        image = [
            i.get('image')
            for i in style
            if i.get('data_value') == k
        ]
        if image:
            return image[0]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2021/8/2 17:46
# @Author  : Adyan
# @File    : Mongo_conn.py
from datetime import datetime

from pymongo import MongoClient


class MongoConn(object):
    def __init__(self, db_name, config):
        """
        :param db_name:
        :param config: {
            "host": "192.168.20.211",
            # "host": "47.107.86.234",
            "port": 27017
            }
        """
        self.db = MongoClient(**config, connect=True)[db_name]


class DBBase(object):

    def __init__(self, collection, db_name, config):
        self.mg = MongoConn(db_name, config)
        self.collection = self.mg.db[collection]

    def exist_list(self, data, key, get_id: callable):

        lst = [get_id(obj) for obj in data]
        print('lst', len(lst))
        set_list = set([
            i.get(key)
            for i in list(
                self.collection.find({key: {"$in": lst}})
            )
        ])
        set_li = set(lst) - set_list
        with open("./ignore/null_field.txt", "rt", encoding="utf-8") as f:
            _ignore = [int(line.split(",")[0]) for line in f.readlines()]
        exist = list(set_li - set(_ignore))
        print(len(exist))
        for obj in data:
            if get_id(obj) in exist:
                yield obj

    def exist(self, dic):
        """
        单条查询
        :param dic:
        :return:1,0
        """
        return self.collection.find(dic).count()

    def update_one(self, dic, item=None):
        result = self.exist(dic)
        if item and result == 1:
            item['updateTime'] = datetime.strftime(datetime.now(), "%Y-%m-%d %H:%M:%S")
            self.collection.update(dic, {"$set": item})
        elif item:
            self.collection.update(dic, {"$set": item}, upsert=True)

    def insert_one(self, param):
        """
        :param param: 多条list 或者 单条dict
        :return:
        """
        self.collection.insert(param)

    def find_len(self, dic):
        return self.collection.find(dic).count()

    def find_one(self):
        return self.collection.find_one()

    def find_list(self, count, dic=None, page=None, ):
        """
        查询数据
        :param count:查询量
        :param dic:{'city': ''} 条件查询
        :param page:分页查询
        :return:
        """
        if dic:
            return list(self.collection.find(dic).limit(count))
        if page:
            return list(self.collection.find().skip(page * count - count).limit(count))

    def daochu(self):
        return list(self.collection.find({'$and': [
            {'$or': [{"transaction_medal": "A"}, {"transaction_medal": "AA"}]},
            {"tpServiceYear": {'$lte': 2}},
            {"overdue": {'$ne': "店铺已过期"}},
            {"province": "广东"}
        ]}))

    # return self.collection.find().skip(count).next()
    def test(self):
        return self.collection


class MongoPerson(DBBase):
    def __init__(self, table, db_name, config):
        super(MongoPerson, self).__init__(table, db_name, config)
